"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const checkDatastore_1 = __importDefault(require("./utility/checkDatastore"));
const getIdPropertyFromSchema_1 = __importDefault(require("./utility/getIdPropertyFromSchema"));
const Core_1 = __importStar(require("./Core"));
const DatastoreSave_1 = __importDefault(require("./operations/DatastoreSave"));
const DatastoreLoad_1 = __importDefault(require("./operations/DatastoreLoad"));
const DatastoreDelete_1 = __importDefault(require("./operations/DatastoreDelete"));
const extractAncestorPaths_1 = __importDefault(require("./utility/extractAncestorPaths"));
const Messaging_1 = require("./Messaging");
const DatastoreQuery_1 = require("./operations/DatastoreQuery");
const DatastoreFlush_1 = __importDefault(require("./operations/DatastoreFlush"));
class PebblebedModel {
    constructor(entityKind, entitySchema, { defaultCachingSeconds = null, neverCache = false, defaultNamespace = Core_1.UNSET_NAMESPACE, } = {}) {
        this.hasIdProperty = false;
        this.defaultCachingSeconds = null;
        this.neverCache = false;
        this.defaultNamespace = Core_1.UNSET_NAMESPACE;
        this.getJoiSchema = () => {
            return this.joiSchema.__getJoiSchema();
        };
        this.validate = (data) => {
            /*const validation = Core.Joi.validate(data, this.joiSchema.__getJoiSchema(), {
              abortEarly: false,
            });*/
            const validation = this.joiSchema.__getJoiSchema().validate(data, { abortEarly: false });
            if (validation.error != null) {
                return {
                    positive: false,
                    message: `[Validation] ${this.entityKind} - ${validation.error.message}`,
                };
            }
            return {
                positive: true,
                message: `[Validation] ${this.entityKind} - Entity data is valid`,
            };
        };
        if (entitySchema.__isPebblebedJoiSchema) {
            this.schema = entitySchema.__generateBasicSchema();
            this.joiSchema = entitySchema;
        }
        else {
            this.schema = entitySchema;
        }
        this.kind = entityKind;
        this.idProperty = (0, getIdPropertyFromSchema_1.default)(this.schema);
        this.defaultCachingSeconds = defaultCachingSeconds;
        this.defaultNamespace = defaultNamespace;
        this.neverCache = neverCache;
        if (this.idProperty != null) {
            this.hasIdProperty = true;
            this.idType = this.schema[this.idProperty].type;
            if (this.idType !== "string" && this.idType !== "int") {
                (0, Messaging_1.throwError)(Messaging_1.CreateMessage.OPERATION_SCHEMA_ID_TYPE_ERROR(this, "CREATE MODEL"));
            }
        }
    }
    save(data) {
        (0, checkDatastore_1.default)("SAVE");
        return new DatastoreSave_1.default(this, data);
    }
    load(idsOrKeys) {
        (0, checkDatastore_1.default)("LOAD");
        return new DatastoreLoad_1.default(this, idsOrKeys);
    }
    query(namespace = Core_1.UNSET_NAMESPACE) {
        (0, checkDatastore_1.default)("QUERY");
        let ns = namespace !== Core_1.UNSET_NAMESPACE ? namespace : this.defaultNamespace;
        return (0, DatastoreQuery_1.createDatastoreQuery)(this, ns);
    }
    key(id) {
        (0, checkDatastore_1.default)("CREATE KEY");
        return Core_1.default.Instance.dsModule.key([this.kind, id]);
    }
    delete(data) {
        (0, checkDatastore_1.default)("DELETE");
        return new DatastoreDelete_1.default(this, data);
    }
    flush(idsOrKeys) {
        (0, checkDatastore_1.default)("FLUSH IN CACHE");
        return new DatastoreFlush_1.default(this, idsOrKeys);
    }
    allocateIds({ amount, withAncestors = null, namespace = Core_1.UNSET_NAMESPACE, }) {
        return __awaiter(this, void 0, void 0, function* () {
            (0, checkDatastore_1.default)("ALLOCATE IDS");
            let ns = namespace !== Core_1.UNSET_NAMESPACE ? namespace : this.defaultNamespace;
            ns = ns !== Core_1.UNSET_NAMESPACE ? ns : (Core_1.default.Instance.namespace !== Core_1.UNSET_NAMESPACE ? Core_1.default.Instance.namespace : null);
            let keyPath = [this.kind];
            if (withAncestors != null) {
                keyPath = [].concat(...(0, extractAncestorPaths_1.default)(this, ...withAncestors), keyPath);
            }
            if (ns != null) {
                const allocateIds = yield Core_1.default.Instance.dsModule.allocateIds(Core_1.default.Instance.dsModule.key({
                    namespace: ns,
                    path: keyPath
                }), amount);
                return allocateIds[0];
            }
            const allocateIds = yield Core_1.default.Instance.dsModule.allocateIds(Core_1.default.Instance.dsModule.key(keyPath), amount);
            return allocateIds[0];
        });
    }
    get entityKind() {
        return this.kind;
    }
    get entitySchema() {
        return this.schema;
    }
    get entityIdProperty() {
        return this.idProperty;
    }
    get entityIdType() {
        return this.idType;
    }
    get entityHasIdProperty() {
        return this.hasIdProperty;
    }
    get entityPebbleSchema() {
        return this.joiSchema;
    }
    get entityDefaultNamespace() {
        return this.defaultNamespace;
    }
    get modelOptions() {
        return {
            defaultCachingSeconds: this.defaultCachingSeconds,
            neverCache: this.neverCache,
        };
    }
}
exports.default = PebblebedModel;
//# sourceMappingURL=PebblebedModel.js.map